using System;
using Gtk;
using Glade;

namespace Amic{
	namespace GridCreator
	{
	
		public class EditPictDialog : Dialog
		{
			
			[Widget] Gtk.Dialog EditPict;
			[Widget] Gtk.Entry InputTextName;
			
			[Widget] Gtk.ComboBox ComboBoxLang; // Language selector
			[Widget] Gtk.Button BtImgSel;    // Image button selector
			[Widget] Gtk.ColorButton BtnBackground; // Fons

			[Widget] Gtk.Entry InputTextCaption;
			[Widget] Gtk.FontButton btnFont;  // Font Selector Button
			[Widget] Gtk.ColorButton BtnFontColor; // Font Color Selector Button
			
			RadioButton RBTop;
			RadioButton RBBottom;
			
			RadioButton RBSpeech;
			RadioButton RBSound;
			RadioButton RBOpen;
			RadioButton RBBack;
		
			[Widget] HBox BoxRBTop;
			[Widget] HBox BoxRBBottom;
			
			[Widget] HBox BoxSpeech;
			[Widget] HBox BoxSound;
			[Widget] HBox BoxOpen;
			[Widget] HBox BoxBack;
		
			[Widget] Gtk.Entry InputTextToSpeech;
			[Widget] Gtk.Button BtSpeech;
			
			[Widget] Gtk.Button BtSndSel;
			[Widget] Gtk.Button BtSound;
			[Widget] Gtk.ComboBox InputComboTheme;
			[Widget] Gtk.ComboBox ComboBoxGrid;
			[Widget] Gtk.ComboBox ComboBoxProjects;
			
			[Widget] Gtk.Button BtOk;
			[Widget] Gtk.Button BtCancel;
			
			// Previsualització
		
			[Widget] Gtk.DrawingArea Preview;
			
			
			// Variables per a la reproducció d'àudio...
			int id_audio=-1;
			IAudioServer server;
			
			// Imatge
			
			Gtk.Image ImageToShow;
			string img_source;
			
			Amic.GridCreator.Pictogram pic;

			public static string SelectedImageFile {get;set;}
			public static string SelectedSoundFile {get;set;}
			
			// Nom del projecte per als comboboxes
			string ProjectName;

			
			public EditPictDialog (GridCreator.Pictogram pic,Gtk.ListStore ThemeStore, string CurrentTheme)
			{	
				Application.Init ();
				
				Glade.XML gxml = new Glade.XML(null, "EditPict.glade", "EditPict", null);
				gxml.Autoconnect (this);
				
				ProjectName=GridCreator.Core.getCore().CurrentProjectName;
				// INICIALITZACIÓ DE L'AUDIO
				NDesk.DBus.ObjectPath path = new NDesk.DBus.ObjectPath("/net/Lliurex/Media/AudioServer");
				server = NDesk.DBus.Bus.Session.GetObject<IAudioServer>("net.Lliurex.Media.AudioServer",path);
				
				GridCreator.Core.getCore().PictoHasBeenModified=false;
				
				// Inicialitzacions
				this.pic=new Amic.GridCreator.Pictogram();
				this.pic=pic;
								
				// Inicialitzem la imatge a mostrar
				
				ImageToShow=new Image();
				
				/******************************************
				 *  Assign initial values
				 * ***************************************/
				// Name
				InputTextName.Text=pic.pictname;
				
				// Language selector
				InitComboLang(pic.lang);
				// Image Button
				BtImgSel.Image=new Image(Stock.File, IconSize.Button);
				BtImgSel.ImagePosition=PositionType.Right;
				// Background
				Gdk.Color BackCol = new Gdk.Color(255, 255, 255);
				Gdk.Color.Parse(pic.background , ref BackCol);
				BtnBackground.Color=BackCol;
				// Caption
				InputTextCaption.Text=pic.caption;
				// Font
				btnFont.SetFontName(pic.fontFace+" "+pic.fontTypeFace+" "+pic.fontSize);
				// Font Color
				Gdk.Color FontCol = new Gdk.Color();
				Gdk.Color.Parse(pic.fontColor , ref FontCol);
				BtnFontColor.Color=FontCol;
				// Sound/Speech Actions
				BtSndSel.Label=pic.actionSound;
				InputTextToSpeech.Text=pic.actionSpeech;
				// RadioButtons
				
				RBTop = new RadioButton  (null, "Top");
            	BoxRBTop.PackStart(RBTop, true, true, 0);
				if (pic.align=="top") RBTop.Active = true;
            	RBTop.Show();
			
				RBBottom = new RadioButton(RBTop, "Bottom");
            	BoxRBBottom.PackStart(RBBottom, true, true, 0);
				if (pic.align!="top") RBBottom.Active = true;
            	RBBottom.Show();
				
				
				// Ocultem els elements que no s'usen
				
				InputTextToSpeech.Sensitive=false;
				BtSndSel.Sensitive=false;
				ComboBoxGrid.Sensitive=false;
				ComboBoxProjects.Sensitive=false;
				
				// RadioButtons de Action
				
				RBSpeech = new RadioButton (null, "Speech Text");
				BoxSpeech.PackStart(RBSpeech, true, true, 0);
				if(pic.actionType=="speech") {
					RBSpeech.Active=true;
					InputTextToSpeech.Sensitive=true;
				}
				RBSpeech.Show();
				
				RBSound= new RadioButton (RBSpeech, "Play Sound");
				BoxSound.PackStart(RBSound, true, true, 0);
				if(pic.actionType=="sound") {
					RBSound.Active=true;
					BtSndSel.Sensitive=true;
				}
				RBSound.Show();
				
				RBOpen = new RadioButton (RBSpeech, "Open a Grid");
				BoxOpen.PackStart(RBOpen, true, true, 0);
				if(pic.actionType=="child") {
					RBOpen.Active=true;
					ComboBoxGrid.Sensitive=true;
					ComboBoxProjects.Sensitive=true;
				}
				RBOpen.Show();
				
				RBBack = new RadioButton (RBSpeech, "Go Back");
				BoxBack.PackStart(RBBack, true, true, 0);
				if(pic.actionType=="parent") RBBack.Active=true; 
				RBBack.Show();
				
				// ComboTheme
				InitComboTheme(ThemeStore, CurrentTheme);
				
				RefreshImageTheme(pic);
				
				// Buttons
				AddImageToButton("speech.png", Stock.MediaPlay, BtSpeech, 16, 16);			
				AddImageToButton("play.png", Stock.MediaPlay, BtSound, 16, 16);
				
				
				InitComboOpenChild(pic.actionChild);
				InitComboOpenChildFromProject(pic.actionChild);
								
				
				/****************************************
				 * End Assign Initial Values
				 * *************************************/
				
				// Event Listeners	
				BtCancel.Clicked += HandleBtCancelClicked;
				BtOk.Clicked += HandleBtOkClicked;
				Preview.ExposeEvent += HandlePreviewExposeEvent;
				
				// Per modificar el preview si canvia el text o els colors...
				InputTextCaption.Changed += HandleInputTextCaptionChanged;
				btnFont.FontSet += HandleBtnFontFontSet;
				BtnFontColor.ColorSet += HandleBtnFontColorColorSet;
				BtnBackground.ColorSet += HandleBtnBackgroundColorSet;
				RBTop.Toggled += HandleRBTopToggled;
				RBBottom.Toggled += HandleRBBottomToggled;
				
				RBSound.Toggled += HandleRBSoundToggled;
				RBSpeech.Toggled += HandleRBSpeechToggled;
				RBOpen.Toggled += HandleRBOpenToggled;
				RBBack.Toggled += HandleRBBackToggled;
				
				BtImgSel.Clicked += HandleBtImgSelClicked;
				BtSndSel.Clicked += HandleBtSndSelClicked;
				
				BtSound.Clicked += HandleBtSoundClicked;
				BtSpeech.Clicked += HandleBtSpeechClicked;
				
				ComboBoxProjects.Changed += HandleComboBoxProjectsChanged;
				
				
				
			}

			void HandleComboBoxProjectsChanged (object sender, EventArgs e)
			{
				string ProjectGrid=((ComboBox)sender).ActiveText;
				Console.WriteLine("AAAA>"+ProjectGrid);
				InitComboOpenChild(ProjectGrid);				
			}
			
			void InitComboOpenChild(string Child){
				// Open Grid Combobox
			
				Console.WriteLine("Child:"+Child);
				string MyProject="";
				string MyGrid="";
				if(Child==null||Child==""){
					MyProject=ProjectName;
					MyGrid="";
				} else{
					string [] ProjectGrid=Child.Split(new Char [] {'/'});
					if(ProjectGrid.Length>1){
						MyProject=ProjectGrid[0];
						MyGrid=ProjectGrid[1];
					} else if (ProjectGrid.Length==1) { // Si és només per refrescar...
						MyProject=ProjectGrid[0];
						MyGrid="";
					} else {
						MyProject=ProjectName;
						MyGrid="";
					}
				}
				
				Console.WriteLine(MyProject);
				Console.WriteLine(MyGrid);
				
				ComboBoxGrid.Clear();
				Gtk.CellRendererText cell=new Gtk.CellRendererText();
				ComboBoxGrid.PackStart(cell, true);
				ComboBoxGrid.AddAttribute(cell, "text", 0);
				
				Gtk.ListStore options_store=new Gtk.ListStore(typeof(string));
				
				string base_grids=GridCreator.Preferences.dir_grids+"/"+MyProject+".AmicProject/";
			
				
				string[] file_list=System.IO.Directory.GetFiles(base_grids);

				int selected=0; int count=0;
				string filename=""; // Nom de l'arxiu sense extensió ni subcarpetes
				foreach(string file in file_list){
					if(file.Substring(file.Length-5, 5)==".grid"){
						filename=file.Replace(base_grids,"").Replace(".grid","");
						options_store.AppendValues(filename);
						if(filename==MyGrid) selected=count;
						count++;
						Console.WriteLine("count: "+count+" selected: "+selected);
						Console.WriteLine("file: "+file+" grid: "+MyGrid+"???");
					}
				}
				ComboBoxGrid.Model=options_store;
				ComboBoxGrid.Active=selected;
				ComboBoxGrid.ShowAll();
			}

			
			void InitComboOpenChildFromProject(string Child){
								
				string MyProject="";
				//string MyGrid="";
				
				if(Child==null||Child==""){
					MyProject=ProjectName;
				//	MyGrid="";
				} else{
					string [] ProjectGrid=Child.Split(new Char [] {'/'});
					if(ProjectGrid.Length>1){
						MyProject=ProjectGrid[0];
					//	MyGrid=ProjectGrid[1];
					} else if (ProjectGrid.Length==1) { // Si és només per refrescar...
						MyProject=ProjectGrid[0];
					//	MyGrid="";
					} else {
						MyProject=ProjectName;
					//	MyGrid="";
					}
				}
								
				ComboBoxProjects.Clear();
				Gtk.CellRendererText cell=new Gtk.CellRendererText();
				ComboBoxProjects.PackStart(cell, true);
				ComboBoxProjects.AddAttribute(cell, "text", 0);
				
				Gtk.ListStore options_store=new Gtk.ListStore(typeof(string));
				
				string base_projects=GridCreator.Preferences.dir_grids+"/";
			
				string[] dir_list=System.IO.Directory.GetDirectories(base_projects, "*.AmicProject");
				
				
				string dirname="";
				int count=0; int selected=0;
				
				foreach(string dir in dir_list){
					dirname=dir.Replace(base_projects,"").Replace(".AmicProject","");
					options_store.AppendValues(dirname);
					if(dirname==MyProject) selected=count;
					count++;
					
					//Console.WriteLine(file.Replace(base_grids,""));
				}
				ComboBoxProjects.Model=options_store;
				ComboBoxProjects.Active=selected;
				ComboBoxProjects.ShowAll();
			}


			
			void HandleBtSpeechClicked (object sender, EventArgs e)
			{
				
				// Creem l'arxiu en funció de l'idioma i el text
				System.IO.FileStream fs=null;
				System.IO.StreamWriter sw=null;
				try{
					if(System.IO.File.Exists("/tmp/saytext.smc"))
						System.IO.File.Delete("/tmp/saytext.smc");
					fs = new System.IO.FileStream("/tmp/saytext.smc", System.IO.FileMode.CreateNew, System.IO.FileAccess.Write);
					sw=new System.IO.StreamWriter(fs);
				
					//Console.WriteLine(ComboBoxLang.ActiveText);
					//Console.WriteLine(this.CurrentPic.lang);
					
					TreeIter Myiter;
					string idioma="en_GB";
					
    	   			ComboBoxLang.Model.GetIterFirst(out Myiter);
	       			do {
	               		GLib.Value thisRow = new GLib.Value ();
						GLib.Value thisRow2 = new GLib.Value ();
               			ComboBoxLang.Model.GetValue (Myiter, 1, ref thisRow);
						//Console.WriteLine("-----------Lang: "+thisRow.Val.ToString());
						ComboBoxLang.Model.GetValue (Myiter, 0, ref thisRow2);
						//Console.WriteLine("Lang: "+thisRow2.Val);
						
				   		if ((thisRow2.Val as string).Equals(ComboBoxLang.ActiveText)) {
								Console.WriteLine("Lang: "+thisRow.Val.ToString());
								idioma=thisRow.Val.ToString();
                       			break;
               			}
						
       				} while (ComboBoxLang.Model.IterNext (ref Myiter));
					
					if(idioma=="qcv_ES")
						if(Preferences.lang_voice=="male")
							sw.WriteLine("(voice_upc_ca_pau_hts)");
							else
							sw.WriteLine("(voice_upc_ca_ona_hts)");
					else if (idioma=="es_ES")
						if(Preferences.lang_voice=="male")
							sw.WriteLine("(voice_JuntaDeAndalucia_es_pa_diphone)");
							else
							sw.WriteLine("(voice_JuntaDeAndalucia_es_sf_diphone)");
					else sw.WriteLine("(voice_kal_diphone)");
				
					
					sw.WriteLine("(SayText \""+InputTextToSpeech.Text+"\")");
					
					sw.Close();
					
					// Execució del festival
					System.Threading.Thread thread;
					thread=null;
					System.Threading.ThreadStart tstart=delegate{
							lliurex.utils.Commands.run("festival", "-b /tmp/saytext.smc");
						};
						
						thread=new System.Threading.Thread(tstart);
						thread.Start();			
				}
				catch (Exception exc){
					Console.WriteLine("Excepcio llegint el text: "+exc);
					if (sw!=null) sw.Close();
				}
			
		
			}

			void HandleBtSoundClicked (object sender, EventArgs e)
			{
				
				string FileToPlay="";
				
				if (System.IO.File.Exists(Preferences.dir_audio_custom+"/"+BtSndSel.Label))
					FileToPlay=Preferences.dir_audio_custom+"/"+BtSndSel.Label;
				else
					FileToPlay=Preferences.dir_audio+"/"+BtSndSel.Label;
			
				Console.WriteLine("Playing... "+FileToPlay);
			
				try{
					//carga
					id_audio = server.LoadMedia(FileToPlay);
					//play
					server.Play(id_audio);
					
				 	
				}catch (Exception exc){
					Console.WriteLine("Exception: "+exc);
				}
			}

			void HandleRBBackToggled (object sender, EventArgs e)
			{
				pic.actionType="parent";
				pic.actionValue="goback";
				
				InputTextToSpeech.Sensitive=false;
				BtSndSel.Sensitive=false;
				ComboBoxGrid.Sensitive=false;
				ComboBoxProjects.Sensitive=false;

			}

			void HandleRBOpenToggled (object sender, EventArgs e)
			{
				pic.actionType="child";
				pic.actionValue=InputTextToSpeech.Text;
				InputTextToSpeech.Sensitive=false;
				BtSndSel.Sensitive=false;
				ComboBoxGrid.Sensitive=true;
				ComboBoxProjects.Sensitive=true;
			}

			void HandleRBSoundToggled (object sender, EventArgs e)
			{
				pic.actionType="sound";
				if (RBSound.Active) SelectSound();
				pic.actionValue=BtSndSel.Label;
				
				InputTextToSpeech.Sensitive=false;
				BtSndSel.Sensitive=true;
				ComboBoxGrid.Sensitive=false;
				ComboBoxProjects.Sensitive=false;
				
			}

			void HandleRBSpeechToggled (object sender, EventArgs e)
			{
				pic.actionType="speech";
				pic.actionValue=InputTextToSpeech.Text;
				
				InputTextToSpeech.Sensitive=true;
				BtSndSel.Sensitive=false;
				ComboBoxGrid.Sensitive=false;
				ComboBoxProjects.Sensitive=false;
				
			}

			
			
			void SelectSound(){
				string path;
			 	SelectedSoundFile=BtSndSel.Label;
			 	try{
				
					//if (this.BtSndSel.Label!=null) path=Preferences.dir_audio+"/"+this.BtSndSel.Label;
					//else path=Preferences.dir_audio;
					
					if (this.BtSndSel.Label!=null) path=Preferences.dir_audio+"/"+this.BtSndSel.Label;
					else path=Preferences.dir_audio+"/Default";
					
					SaveSoundWindow filedlg=new SaveSoundWindow(this.BtSndSel.Label);
					filedlg.Show();
					  
					//if(SelectedSoundFile!=path&&(System.IO.File.Exists(Preferences.dir_audio+"/"+SelectedSoundFile)||
				      //                       	System.IO.File.Exists(Preferences.dir_audio_custom+"/"+SelectedSoundFile))){
					if(SelectedSoundFile!=path&& System.IO.File.Exists(SelectedSoundFile)){
						
						string file="";
						// Restem les rutes a la imatge, per saber si està a Cutom o a Default			
						file=GeneralUtils.GetFilenameFrom(SelectedSoundFile, Preferences.dir_audio_custom, Preferences.dir_audio);
						
						BtSndSel.Label=file;					
						pic.actionSound=file;
						
					} else {
						Console.WriteLine("Ha donat a cancel·lar: path="+path+" SelectedSoundFile: "+SelectedSoundFile);
					}
					
					//this.BtSelectImage.SetFilename(myfile);
				} catch (Exception exc){Console.WriteLine("Exception: "+exc); }
				
			}
			
			void HandleBtSndSelClicked (object sender, EventArgs e)
			{
				SelectSound();
			
			}

	
			void HandleBtImgSelClicked (object sender, EventArgs e)
			{
				
				string path;
				string tema=InputComboTheme.ActiveText;
				
				// Patch perquè aparega el tema per defecte si no hi ha cap triat:
				if(tema==""||tema==null) tema="Default";
				
				try{ // Si no hi ha imatge seleccionada, passem el directori de la galeria per defecte
					if (this.BtImgSel.Label!=null) path=Preferences.dir_img+"/"+this.BtImgSel.Label;
					else path=Preferences.dir_img;
					
					// Li passem al diàleg la imatge corresponent...
					SaveFileWindow filedlg;
					if(pic.image_list.ContainsKey(tema))
						filedlg=new SaveFileWindow(pic.image_list[tema]);
					else
						filedlg=new SaveFileWindow("amic.png");
				
					filedlg.Show();
					Console.WriteLine("path: "+path+" SelectedImage: *"+SelectedImageFile+"*");
					
					// TODO: CAL ACTUALITZAR EL TEXT DEL BOTÓ AMB NOMÉS EL NOM DE LA IMATGE!!!
					
					if(SelectedImageFile!=path&& System.IO.File.Exists(SelectedImageFile)){
						Console.WriteLine("Cal actualitzar la imatge: "+SelectedImageFile);
						
						//BtSelectImage.SetFilename(SelectedImageFile);
						
						//BtImgSel.Label=SelectedImageFile;
						//pic.image_list[tema]=SelectedImageFile;
				
						string file="";
						// Restem les rutes a la imatge, per saber si està a Cutom o a Default
			
						file=GeneralUtils.GetFilenameFrom(SelectedImageFile, Preferences.dir_img_custom, Preferences.dir_img);
						BtImgSel.Label=file;
					
						pic.image_list[tema]=file;
						RefreshImageTheme(pic);
						//DrawPreview();
					} else {
						Console.WriteLine("Ha donat a cancel·lar");
					}
				} catch (Exception ex){
					Console.WriteLine("Exception: "+ex);
				}
			}
			
			
			
			void HandleRBBottomToggled (object sender, EventArgs e)
			{
				this.pic.align="bottom";
				DrawPreview();
			}
	
			void HandleRBTopToggled (object sender, EventArgs e)
			{
				this.pic.align="top";
				DrawPreview();
			}

			void HandleBtnBackgroundColorSet (object sender, EventArgs e)
			{
				if (ImageToShow.Pixbuf!=null) DrawPreview();
			}

			void HandleBtnFontColorColorSet (object sender, EventArgs e)
			{
				if (ImageToShow.Pixbuf!=null) DrawPreview();
			}

			void HandleBtnFontFontSet (object sender, EventArgs e)
			{
				if (ImageToShow.Pixbuf!=null) DrawPreview();
			}

			void HandleInputTextCaptionChanged (object sender, EventArgs e)
			{
				if (ImageToShow.Pixbuf!=null) DrawPreview();
			}

			void HandlePreviewExposeEvent (object o, ExposeEventArgs args)
			{
				if (ImageToShow.Pixbuf!=null) DrawPreview();
			}
			public new void Show(){
				EditPict.ShowAll();
				EditPict.Modal=true;
				EditPict.SetPosition(WindowPosition.CenterAlways);
				Application.Run();
			
				
			}
			
			void HandleBtOkClicked (object sender, EventArgs e)
			{
				
				pic.pictname=InputTextName.Text;
				pic.lang=ComboBoxLang.ActiveText;
				pic.background=TransformColor(BtnBackground.Color.ToString());
				pic.caption=InputTextCaption.Text;
				
				Pango.FontDescription desc=Pango.FontDescription.FromString(btnFont.FontName);
				pic.fontFace=desc.Family;
				pic.fontTypeFace=desc.Weight+" "+desc.Style;
				pic.fontSize=(desc.Size/1000);

				pic.fontColor=TransformColor(BtnFontColor.Color.ToString());
				
				if(RBTop.Active) pic.align="top";
				else pic.align="bottom";
				
				pic.actionSpeech=InputTextToSpeech.Text;
				pic.actionSound=BtSndSel.Label;
				//pic.actionChild=GridCreator.Core.getCore().CurrentProjectName+"/"+ComboBoxGrid.ActiveText;
				pic.actionChild=ComboBoxProjects.ActiveText+"/"+ComboBoxGrid.ActiveText;
				
				if(RBSpeech.Active) {pic.actionType="speech"; pic.actionValue=InputTextToSpeech.Text;}
				else if(RBSound.Active) {pic.actionType="sound"; pic.actionValue=BtSndSel.Label;}
				else if(RBOpen.Active) {pic.actionType="child"; pic.actionValue=InputTextToSpeech.Text;}
				else if(RBBack.Active) {pic.actionType="parent"; pic.actionValue=BtSndSel.Label;}
												
				GridCreator.Core.getCore().PictoModified=pic;
				GridCreator.Core.getCore().PictoHasBeenModified=true;
				GridCreator.Core.getCore().CurrentFileSaved=false; // Diem que no s'ha gravat l'arxiu...
				Application.Quit();
				EditPict.Destroy();
			}	
			
			void HandleBtCancelClicked (object sender, EventArgs e)
			{
				Application.Quit();
				EditPict.Destroy();
			}
			
			
			public void InitComboTheme(Gtk.ListStore store, string CurrentTheme){
				
				InputComboTheme.Clear();
				Gtk.CellRendererText cell=new Gtk.CellRendererText();
				
				InputComboTheme.PackStart(cell, true);
			    InputComboTheme.AddAttribute(cell, "text", 0);
				InputComboTheme.Model=store;
				
				
				/* Per si l'store és buit */
			   	GLib.Value thisRow = new GLib.Value ();
				TreeIter Myiter;
				InputComboTheme.Model.GetIterFirst(out Myiter);
				InputComboTheme.Model.GetValue (Myiter,  0, ref thisRow);
				if (thisRow.Val==null)
				{
					Gtk.ListStore newstore=new Gtk.ListStore(typeof(string));
					newstore.AppendValues("Default");
					InputComboTheme.Model=newstore;
				}
					
				/**/
				
			
				// Deixem l'ítem seleccionat
								
				InputComboTheme.Model.GetIterFirst(out Myiter);
				
												
				do {
	               	thisRow = new GLib.Value ();
               		InputComboTheme.Model.GetValue (Myiter,  0, ref thisRow);
				
					//Console.WriteLine("-----------THeme: "+thisRow.Val.ToString());
				   	if (thisRow.Val!=null && (thisRow.Val as string).Equals(CurrentTheme)) {
                       		InputComboTheme.SetActiveIter(Myiter);
                       		break;
               		}
					
       			} while (InputComboTheme.Model.IterNext (ref Myiter));
								
			}
						
			public void InitComboLang(string lang){
				/*
			 	* Inicialitza el ComboBox dels idiomes
			 	* 
			 	* */
				ListStore store = new ListStore(typeof (string), typeof (string), typeof(string));
				this.ComboBoxLang.Clear(); 	// Combo per triar el tema de visualització
				Gtk.CellRendererText cell=new Gtk.CellRendererText();
				this.ComboBoxLang.PackStart(cell, true);
			    this.ComboBoxLang.AddAttribute(cell, "text", 0);
				this.ComboBoxLang.Model = store;
				
				store.AppendValues( Mono.Unix.Catalog.GetString("Valencian"), "qcv_ES");
				store.AppendValues( Mono.Unix.Catalog.GetString("Spanish"), "es_ES");
				store.AppendValues( Mono.Unix.Catalog.GetString("English"), "en_GB");
							
				TreeIter Myiter;
       			ComboBoxLang.Model.GetIterFirst(out Myiter);
       			do {
	               		GLib.Value thisRow = new GLib.Value ();
               			ComboBoxLang.Model.GetValue (Myiter, 1, ref thisRow);
						//Console.WriteLine("-----------Lang: "+thisRow.Val.ToString());
				   		if ((thisRow.Val as string).Equals(lang)) {
								Console.WriteLine("Lang: "+lang);
                       			ComboBoxLang.SetActiveIter (Myiter);
                       			break;
               			}
       			} while (ComboBoxLang.Model.IterNext (ref Myiter));
				if(ComboBoxLang.ActiveText==""||ComboBoxLang.ActiveText==null) ComboBoxLang.Active=0;
				

				
			}
				
			public void RefreshImageTheme(Amic.GridCreator.Pictogram pic){
				try{
					Image MiImage;
					string tema=InputComboTheme.ActiveText;
					Console.WriteLine("Tema1: "+tema);
					if(tema==""||tema==null) tema="Default";
					Console.WriteLine("Tema2: "+tema);
					
			    	img_source=pic.image_list[tema];
					
					
					Console.WriteLine("source: "+img_source);
					
					if (img_source==""||img_source==null) {
						img_source="undefined.png";
					}
	
					Console.WriteLine("source: "+img_source);
						
					if (System.IO.File.Exists(Preferences.dir_img_custom+"/"+img_source))
					{
						Console.WriteLine("Trobe "+Preferences.dir_img_custom+"/"+img_source);					
						MiImage = new Gtk.Image(Preferences.dir_img_custom+"/"+img_source);
					}else if (System.IO.File.Exists(Preferences.dir_img+"/"+img_source))
					{
						Console.WriteLine("Trobe "+Preferences.dir_img+"/"+img_source);					
						MiImage = new Gtk.Image(Preferences.dir_img+"/"+img_source);
					} else  if (System.IO.File.Exists(Preferences.dir_default+"/notfound.png")){ 
						Console.WriteLine("Trobe "+Preferences.dir_default+"/notfound.png");
						MiImage = new Gtk.Image(Preferences.dir_default+"/notfound.png");
					} else {
						Console.WriteLine("No trobe cap imatge");
						MiImage = new Gtk.Image(Stock.DialogError);
					}
				
					int new_width=125;
					int new_height=125;
     				
					MiImage.Pixbuf = MiImage.Pixbuf.ScaleSimple(new_width, new_height, Gdk.InterpType.Bilinear);
					
					ImageToShow.Pixbuf=MiImage.Pixbuf;					
					
					DrawPreview();
									
					this.BtImgSel.Label=img_source;
					
				}catch(Exception e){
					Console.WriteLine("Exception Refreshing Theme: "+e.Message);
				}
			}
						
			void DrawPreview(){
				try{
					Gtk.Image MyImage=new Gtk.Image();
					MyImage.Pixbuf=ImageToShow.Pixbuf;
					
					Gdk.GC gc=((Gtk.DrawingArea)Preview).Style.TextGC(Gtk.StateType.Normal);
				
					//Cairo.Context context = Gdk.CairoHelper.Create (((Gtk.DrawingArea)obj).GdkWindow);
	
					Cairo.Context context = Gdk.CairoHelper.Create (Preview.GdkWindow);
					
					int daX, daY, x, y;
					//MyImage.GdkWindow.GetSize(out x, out y);
					x=MyImage.Pixbuf.Width;
					y=MyImage.Pixbuf.Height;
					
		    		Preview.GdkWindow.GetSize(out daX, out daY);
			    	
					int posX=(daX/2)-(x/2);
					int posY=(daY/2)-(y/2);
					Console.WriteLine("("+x+","+y+")");
					Console.WriteLine("("+daX+","+daY+")");
					Console.WriteLine("("+posX+","+posY+")");
					
					//context.Color=new Cairo.Color(1,1,0.5);
					double r=double.Parse((BtnBackground.Color.Red).ToString())/65535;
					double g=double.Parse((BtnBackground.Color.Green).ToString())/65535;
					double b=double.Parse((BtnBackground.Color.Blue).ToString())/65535;
					Console.WriteLine("("+BtnBackground.Color.Red+","+BtnBackground.Color.Green+","+BtnBackground.Color.Blue+")");
					Console.WriteLine("("+r+","+g+","+b+")");
					context.Color=new Cairo.Color(r,g,b);
					context.Rectangle(0,0,daX,daY);
				
					Cairo.Gradient pat = new Cairo.LinearGradient (posX, 0, posX, daY);
					pat.AddColorStop (0, new Cairo.Color (r,g,b));
					
					pat.AddColorStop (1, new Cairo.Color (r*1.5,g*1.5,b*1.5));
					context.Pattern=pat;
					context.FillPreserve();
					
					Preview.GdkWindow.DrawPixbuf(gc, MyImage.Pixbuf, 0, 0,posX,posY, x,y, Gdk.RgbDither.Max , 0, 0);
					
					Pango.Layout layout;	
					layout = new Pango.Layout(EditPict.PangoContext);
					layout.Width = Pango.Units.FromPixels(100);
					layout.Wrap = Pango.WrapMode.Word;
					layout.Alignment = Pango.Alignment.Left;
					
					layout.FontDescription = Pango.FontDescription.FromString(btnFont.FontName);
					
					layout.SetMarkup("<span color='"+TransformColor((BtnFontColor.Color).ToString())+"'>" 
			                 		+ InputTextCaption.Text+ "</span>");
				 			    
					int textX, textY;
					layout.GetPixelSize(out textX, out textY);
					
					Console.WriteLine("("+textX+","+textY+")");
					
					
					
					if (RBTop.Active == true) 
						Preview.GdkWindow.DrawLayout (Preview.Style.TextGC (StateType.Normal), daX/2-textX/2, 15, layout);
					else
						Preview.GdkWindow.DrawLayout (Preview.Style.TextGC (StateType.Normal), daX/2-textX/2, daY-15-textY, layout);
		
					
					
					((IDisposable)context.Target).Dispose();
					((IDisposable)context).Dispose();
					
				} catch (Exception e){
					Console.WriteLine("Exception Drawing: "+e.Message);
				}
			}
			
			
			public void AddImageToButton(string image, string stock, Button Boto, int tamX, int tamY){
				// Utilitat per assignar una imatge a un botó.
				Gtk.Image MiImage=new Gtk.Image();
				if (System.IO.File.Exists(Preferences.dir_rsrc+"/"+image))
				{			
					MiImage = new Gtk.Image(Preferences.dir_rsrc+"/"+image);
					if (tamX!=0) MiImage.Pixbuf=MiImage.Pixbuf.ScaleSimple(tamX, tamY, Gdk.InterpType.Bilinear);
					Boto.Image=MiImage;
				} else { 
					MiImage = new Image(stock, IconSize.Button);
					Boto.Image=MiImage;
				}
	
			}

			
			
			public string TransformColor(string gtkcolor){
			/*
		 	* Torna un string tipus #rrggbb a partir d'un gtkcolor
		 	* 
		 	* */
				
			string [] color = gtkcolor.Split(new Char [] {':'});
			if (color.Length<1) return "#000000";
			else{
				string [] components=color[1].Split(new Char [] {'/'});
				if (components.Length!=3) return  "#000000";
				else{
					string r,g,b;
					if (components[0].Length>2) r=components[0].Substring(0,2);
					else r="0"+components[0].Substring(0,1);
					
					if (components[1].Length>2) g=components[1].Substring(0,2);
					else g="0"+components[1].Substring(0,1);
					
					if (components[2].Length>2) b=components[2].Substring(0,2);
					else b="0"+components[2].Substring(0,1);
					
					return "#"+r+g+b;
				}
					
			}
		}
		
		}
	}
	
}
